/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtDesc.c  
*
* DESCRIPTION:                                                                 
*       MPSC and Ethernet descriptors related functions.
*
* DEPENDENCIES:
*
*******************************************************************************/

/* includes */
#include "gtDesc.h"
        
/*******************************************************************************
* gtDescInitRxChain - Initialize a chain of descriptors for RX.
*
* DESCRIPTION:
*           Initialize a chain of descriptors for RX.
*
* INPUT:
*           numOfDesc - number of descriptors in chain                            
*           firstRxDescAddr - address of first desc                            
*           firstRxBufferAddr - address of first buffer                                   
*           rxBufferSize - buffer size (in bytes)  
*           cmdStsWord  - A word to be written in all descs' command word. Use
*                         defines in gtDesc.h to set bits. 
*           command - Create a cyclic chain or not - use defines in gtDesc.h                      
*
* OUTPUT:
*           None.
*
* RETURN:
*           DESC_INIT_STATUS word that determines if the initialization
*           succeeded. In case of failure it determines the exact failure 
*           reason. 
*
*******************************************************************************/
DESC_INIT_STATUS gtDescInitRxChain(unsigned int numOfDesc, 
                                   unsigned int firstRxDescAddr,
                                   unsigned int firstRxBufferAddr,
                                   unsigned int rxBufferSize,
                                   unsigned int cmdStsWord,
				   DESC_COMMAND command)
{
    unsigned int    i;
    unsigned int    bufferAddr;
    RX_DESC*        rxPointer;
    bool            cyclic = false;
    /* mask firstRxBufferAddr except 3 lsb */
    i = (firstRxBufferAddr & (0x00000007));     
    if(i != 0x0)
    {
        return BUFFERS_BASE_NOT_ALLINGED;
    }
    i = (firstRxDescAddr & (0x0000000f));     
    if(i != 0x0)
    {
        return DESCS_BASE_NOT_ALLINGED;
    }
    if(rxBufferSize > 0x10000)
    {
        return BUFFER_TOO_BIG;
    }
    
    if(command & CREATE_CYCLIC_CHAIN)
    {
        cyclic = true;
    }

    rxPointer = (RX_DESC*)PHY_TO_VIRTUAL(firstRxDescAddr);
    bufferAddr = PHY_TO_VIRTUAL(firstRxBufferAddr);
    for(i = 0; i < numOfDesc; i++)
    {
        if((bufferAddr & 0x00000007) != 0x0)
        {
          /* round up Address to closest allinged address */
          bufferAddr = bufferAddr + 8 - (bufferAddr & 0x00000007);  
        }
        rxPointer[i].cmd_sts =  cmdStsWord; 
        /* buffer size */
        rxPointer[i].bufsize = rxBufferSize;
        /* initialize counter */
        rxPointer[i].bytecnt = 0;    
        /* buffer pointer */
        rxPointer[i].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        rxPointer[i].next_desc_ptr = VIRTUAL_TO_PHY(firstRxDescAddr + (i+1) * 
                                                    DESC_SIZE);
        bufferAddr += rxBufferSize;
    }

    if (cyclic)
    {
        /* make the descriptors chain cyclic */
        rxPointer[i-1].next_desc_ptr = VIRTUAL_TO_PHY(firstRxDescAddr);
    }
    else
    {
        rxPointer[i-1].next_desc_ptr = 0;
    }

    return DESC_INIT_SUCCESS;
}


/*******************************************************************************
* gtDescInitTxChain - Initialize a chain of descriptors for TX.
*
* DESCRIPTION:
*          Iinitialize a chain of descriptors for TX.
*
* INPUT:
*           numOfDesc - num of descriptors in chain                            
*           firstTxDescAddr - address of first desc                            
*           firstTxBufferAddr - address of first buffer                                   
*           txBufferSize - buffer size (in bytes)  
*           cmdStsWord  - A word to be written in all descs' command word. Use
*           defines in gtDesc.h to set bits. 
*           command - Create a cyclic chain or not - use defines in gtDesc.h                      
*
* OUTPUT:
*           None.
*
* RETURN:
*           DESC_INIT_STATUS word that determines if the initialization
*           succeeded. In case of failure it determines the exact failure 
*           reason. 
*
*******************************************************************************/
DESC_INIT_STATUS gtDescInitTxChain(unsigned int numOfDesc, 
                                   unsigned int firstTxDescAddr,
                                   unsigned int firstTxBufferAddr,
                                   unsigned int txBufferSize,
                                   unsigned int cmdStsWord,
					         	   DESC_COMMAND command, 
                                   unsigned int numOfBufPerPacket)
{

    unsigned int    i;
    unsigned int    bufferAddr;
    TX_DESC*        txPointer;
    bool            cyclic = false;
    
    /* The following check is WWWWWRRRRRRRRROOOONNNNNNNNNNGGGGGGG - Rabeeh */
    /*    i = (firstTxBufferAddr & (0x00000007));     
	  if(i != 0x0)
	  {
	  return BUFFERS_BASE_NOT_ALLINGED;
	  }*/
    
    i = (firstTxDescAddr & (0x0000000f));     
    if(i != 0x0)
    {
        return DESCS_BASE_NOT_ALLINGED;
    }
    if(txBufferSize > 0x10000)
    {
        return BUFFER_TOO_BIG;
    }

    if(numOfBufPerPacket == 0 || numOfBufPerPacket > numOfDesc)
    {
        return WRONG_NUM_OF_BUFFERS_PER_PACKET;
    }
    
    if(command & CREATE_CYCLIC_CHAIN)
    {                         
        cyclic = true;
    }

    txPointer = (TX_DESC*)PHY_TO_VIRTUAL(firstTxDescAddr);
    bufferAddr = PHY_TO_VIRTUAL(firstTxBufferAddr);
    for(i = 0; i < numOfDesc; i++)
    {
      /* The following check is WRONG - Rabeeh */
      //        if((bufferAddr & 0x00000007) != 0x0)
      //        {
      //            /* round up Address to closest allinged address */
      //            bufferAddr = bufferAddr + 8 - (bufferAddr & 0x00000007);  
      //        }
        txPointer[i].cmd_sts =  cmdStsWord;
        /* buffer size */
        txPointer[i].bytecnt = txBufferSize;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i].shadow = txBufferSize;
        /* buffer pointer */
        txPointer[i].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + 
                                                    (i+1)*DESC_SIZE);
        bufferAddr += txBufferSize;
    }
    if (cyclic)
    {
        /* make the descriptors chain cyclic */
        txPointer[i-1].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr);
    }
    else
    {
        txPointer[i-1].next_desc_ptr = 0;
    }

    /* set First and last flags */
    txPointer = (TX_DESC*)PHY_TO_VIRTUAL(firstTxDescAddr);
    for(i = 0 ; i < numOfDesc ; i+=numOfBufPerPacket)
    {
        txPointer[i].cmd_sts = txPointer[i].cmd_sts | DESC_FIRST;
        if(i + numOfBufPerPacket - 1 > numOfDesc - 1)
        {
            txPointer[numOfDesc - 1].cmd_sts = 
                txPointer[numOfDesc - 1].cmd_sts | DESC_LAST;
            break;
        }
        txPointer[i + numOfBufPerPacket - 1].cmd_sts = 
            txPointer[i + numOfBufPerPacket - 1].cmd_sts | DESC_LAST;
    }
    return DESC_INIT_SUCCESS;
}
/*******************************************************************************
* gtDescGetTxOwnership - returns desc ownership
*
* DESCRIPTION:
*           Checks ownership bit in desc status word to determine its ownership.      
*
* INPUT:
*      descAddr - address of the tx desc to be checked.
*
* OUTPUT:
*      None.
*
* RETURN:
*     DESC_OWNER_IS_GT - if owned by gt. 
*     DESC_OWNER_IS_CPU - if owned by cpu. 
*
*******************************************************************************/
DESC_OWNER gtDescGetTxOwnership(unsigned int descAddr)
{
    TX_DESC*     desc = (TX_DESC*)PHY_TO_VIRTUAL(descAddr);
    unsigned int cmd_sts;

    cmd_sts = desc->cmd_sts;
    if(cmd_sts & DESC_OWNED_BY_GT)
    {
        return DESC_OWNER_IS_GT;
    }
    return DESC_OWNER_IS_CPU;
}

/*******************************************************************************
* gtDescGetRxOwnership - returns desc ownership
*
* DESCRIPTION:
*           Checks ownership bit in desc status word to determine its ownership.      
*
* INPUT:
*      descAddr - address of the rx desc to be checked.
*
* OUTPUT:
*      None.
*
* RETURN:
*     DESC_OWNER_IS_GT - if owned by gt. 
*     DESC_OWNER_IS_CPU - if owned by cpu. 
*
*******************************************************************************/
DESC_OWNER gtDescGetRxOwnership(unsigned int descAddr)
{
    RX_DESC*     desc = (RX_DESC*)PHY_TO_VIRTUAL(descAddr);
    unsigned int cmd_sts;

    cmd_sts = desc->cmd_sts;
    if(cmd_sts & DESC_OWNED_BY_GT)
    {
        return DESC_OWNER_IS_GT;
    }
    return DESC_OWNER_IS_CPU;
}

/******************************************************************************* 
* gtDescSetTxOwnedByGT - move all desc in chain to gt ownership
*
* DESCRIPTION:
*       Set ownership bit in all tx desc in chain.
*
* INPUT:
*       firstTxDescAddr - address of first descriptor in chain to be moved to gt 
*                         ownership.
*       numOfDesc - number of descriptors to be moved to gt ownership
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtDescSetTxOwnedByGT(unsigned int firstTxDescAddr, unsigned int numOfDesc)
{
    int      i;
    TX_DESC* txPointer = (TX_DESC*)PHY_TO_VIRTUAL(firstTxDescAddr);
    for(i = 0; i < numOfDesc; i++)
    {
        txPointer->cmd_sts = txPointer->cmd_sts | DESC_OWNED_BY_GT;
        txPointer = (TX_DESC*)PHY_TO_VIRTUAL(txPointer->next_desc_ptr); 
    }
}


/******************************************************************************* 
* gtDescSetRxOwnedByGT - move all desc in chain to gt ownership
*
* DESCRIPTION:
*       Set ownership bit in all rx desc in chain.
*
* INPUT:
*       firstRxDescAddr - address of first descriptor in chain to be moved to gt 
*                         ownership.
*       numOfDesc - number of descriptors to be moved to gt ownership.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtDescSetRxOwnedByGT(unsigned int firstRxDescAddr,unsigned int numOfDesc)
{
    int      i;
    RX_DESC* rxPointer = (RX_DESC*)PHY_TO_VIRTUAL(firstRxDescAddr);
    for(i = 0; i < numOfDesc; i++)
    {
        rxPointer->cmd_sts = rxPointer->cmd_sts | DESC_OWNED_BY_GT;
        rxPointer = (RX_DESC*)PHY_TO_VIRTUAL(rxPointer->next_desc_ptr); 
    }
}


/*******************************************************************************
* gtDescIsOverRunSet - checks if overRun event has happened
*
* DESCRIPTION:
*       Checks in a chain of rx descriptors if OverRun bit is set in any of the
*       chains' descriptors status word        
*
* INPUT:
*       firstRxDescAddr - first desc to be checked.
*       numOfDesc - number of descs to be checked.
* OUTPUT:
*       None.
*
* RETURN:
*       true - if underRun bit is set in any of the descs.
*       false - otherwise.
*******************************************************************************/
bool gtDescIsOverRunSet(unsigned int firstRxDescAddr,
                        unsigned int numOfDesc)
{
    int      i;
    RX_DESC* rxPointer = (RX_DESC*)PHY_TO_VIRTUAL(firstRxDescAddr);
    for(i = 0; i < numOfDesc; i++)
    {
        if(rxPointer->cmd_sts & DESC_DATA_OVERRUN) return true;
        rxPointer = (RX_DESC*)PHY_TO_VIRTUAL(rxPointer->next_desc_ptr); 
    }
    return false;
}



/*******************************************************************************
* gtDescIsUnderRunSet - checks if underRun event has happened
*
* DESCRIPTION:
*       Checks in a chain of tx descriptors if underRun bit is set in any of the
*       chains' descriptors status word        
*
* INPUT:
*       firstTxDescAddr - first desc to be checked.
*       numOfDesc - number of descs to be checked.
*
* OUTPUT:
*       None.
*
* RETURN:
*       true - if underRun bit is set in any of the descs.
*       false - otherwise.
*******************************************************************************/
bool gtDescIsUnderRunSet(unsigned int firstTxDescAddr,
                         unsigned int numOfDesc)
{
    int      i;
    TX_DESC* txPointer = (TX_DESC*)PHY_TO_VIRTUAL(firstTxDescAddr);
    for(i = 0; i < numOfDesc; i++)
    {
        if(txPointer->cmd_sts & DESC_DATA_UNDERRUN) return true;
        txPointer = (TX_DESC*)PHY_TO_VIRTUAL(txPointer->next_desc_ptr); 
    }
    return false;
}

/*******************************************************************************
* gtDescInitTxChainForBisync - initialize a chain of descriptors for TX.
*
* DESCRIPTION:
*           Initialize a chain of descriptors for TX (only for bisync).
*
* INPUT:
*           bisyncMode - BISYNC_NORMAL,
*						 BISYNC_TRANSPARENT,
*						 BISYNC_AUTO_TRANSPARENT
*			numOfDesc - num od descriptors in chain.                            
*           firstTxDescAddr - address pf first desc.                            
*           firstTxBufferAddr - address of first buffer                                   
*           txBufferSize - buffer size (in bytes)  
*           cmdStsWord  - a word to be written in all descs' command word. use
*           defines in gtDesc.h to set bits. 
*           command - create a cyclic chain or not - use defines in gtDesc.h                      
*
* OUTPUT:
*           None.
*
* RETURN:
*           DESC_INIT_STATUS word that determines if the initialization
*           succeeded. In case of failure it determines the exact failure 
*           reason. 
*
*******************************************************************************/
DESC_INIT_STATUS gtDescInitTxChainForBisync(unsigned int bisyncMode,
											unsigned int numOfDesc, 
                                            unsigned int firstTxDescAddr,
                                            unsigned int firstTxBufferAddr,
                                            unsigned int txBufferSize,
                                            unsigned int cmdStsWord,
								            DESC_COMMAND command)
{

    unsigned int    i;
    unsigned int    bufferAddr;
    TX_DESC*        txPointer;
    bool            cyclic = false;
    
    i = (firstTxBufferAddr & (0x00000007));     
    if(i != 0x0)
    {
        return BUFFERS_BASE_NOT_ALLINGED;
    }
    i = (firstTxDescAddr & (0x0000000f));     
    if(i != 0x0)
    {
        return DESCS_BASE_NOT_ALLINGED;
    }
    if(txBufferSize > 0x10000)
    {
        return BUFFER_TOO_BIG;
    }

    if(command & CREATE_CYCLIC_CHAIN)
    {                         
        cyclic = true;
    }

    txPointer = (TX_DESC*)(firstTxDescAddr);
    bufferAddr = firstTxBufferAddr;
	
	/* Set the descriptors for BISYNC protocol. each original descriptor is
	   replace by three descriptors first for SOT and second for DATA 
	   and third for EOT */

    for(i = 0; i < 3 * numOfDesc; i=i+3)
    {
        if((bufferAddr & 0x00000007) != 0x0)
        {
            /* round up Address to closest allinged address */
            bufferAddr = bufferAddr + 8 - (bufferAddr & 0x00000007);  
        }
		/* Set first bisync descriptor */
		if(bisyncMode == BISYNC_NORMAL)
        {
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST | DESC_RESET_BCC;

        }

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST |
                                    DESC_TRANSMIT_DLE_BEFORE_BUFFER;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST | 
									DESC_TRANSMIT_DLE_BEFORE_BUFFER;
        /* buffer size */
        txPointer[i].bytecnt = 1;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i].shadow = 1;
        /* buffer pointer */
        txPointer[i].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+1) *
                                                    DESC_SIZE);
        bufferAddr += 8;

		/* Set second bisync descriptor */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

        /* buffer size */
        txPointer[i+1].bytecnt = txBufferSize;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+1].shadow = txBufferSize;
        /* buffer pointer */
        txPointer[i+1].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+1].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + 
                                                    (i+2)*DESC_SIZE);
        bufferAddr += txBufferSize;

		/* Set third bisync descriptor */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
								      DESC_GENERATE_ERROR_CODE_TX | DESC_LAST;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_GENERATE_ERROR_CODE_TX | DESC_LAST |
									  DESC_TRANSMIT_DLE_BEFORE_BUFFER;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_GENERATE_ERROR_CODE_TX | DESC_LAST |
									  DESC_TRANSMIT_DLE_BEFORE_BUFFER;
        /* buffer size */
        txPointer[i+2].bytecnt = 1;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+2].shadow = 1;
        /* buffer pointer */
        txPointer[i+2].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+2].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+3) *
                                                      DESC_SIZE);
        bufferAddr += 8;
    }

    if (cyclic)
    {
        /* make the descriptors chain cyclic */
        txPointer[i-1].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr);
    }
    else
    {
        txPointer[i-1].next_desc_ptr = 0;
    }

    return DESC_INIT_SUCCESS;
}



/*******************************************************************************
* gtDescInitTxChainForBisyncHeader - initialize a chain of descriptors for TX.
*									 with headers.
* DESCRIPTION:
*           Initialize a chain of descriptors for TX at bisync only.
*
* INPUT:
*           bisyncMode - BISYNC_NORMAL,
*						 BISYNC_TRANSPARENT,
*						 BISYNC_AUTO_TRANSPARENT
*			numOfDesc - num od descriptors in chain                            
*           firstTxDescAddr - address pf first desc                            
*           firstTxBufferAddr - address of first buffer                                   
*           txBufferSize - buffer size (in bytes)  
*           cmdStsWord  - a word to be written in all descs' command word. use
*           defines in gtDesc.h to set bits. 
*           command - create a cyclic chain or not - use defines in gtDesc.h                      
*
* OUTPUT:
*           None.
*
* RETURN:
*           DESC_INIT_STATUS word that determines if the initialization
*           succeeded. In case of failure it determines the exact failure 
*           reason. 
*s
*******************************************************************************/
DESC_INIT_STATUS gtDescInitTxChainForBisyncHeader(unsigned int bisyncMode,
											unsigned int numOfDesc, 
                                            unsigned int firstTxDescAddr,
                                            unsigned int firstTxBufferAddr,
                                            unsigned int txBufferSize,
                                            unsigned int cmdStsWord,
								            DESC_COMMAND command)
{

    unsigned int    i;
    unsigned int    bufferAddr;
    TX_DESC*        txPointer;
    bool            cyclic = false;
    
    i = (firstTxBufferAddr & (0x00000007));     
    if(i != 0x0)
    {
        return BUFFERS_BASE_NOT_ALLINGED;
    }
    i = (firstTxDescAddr & (0x0000000f));     
    if(i != 0x0)
    {
        return DESCS_BASE_NOT_ALLINGED;
    }
    if(txBufferSize > 0x10000)
    {
        return BUFFER_TOO_BIG;
    }

    if(command & CREATE_CYCLIC_CHAIN)
    {                         
        cyclic = true;
    }

    txPointer = (TX_DESC*)(firstTxDescAddr);
    bufferAddr = firstTxBufferAddr;
	
	/* Set the descriptors for BISYNC protocol. each original descriptor is
	   replace by three descriptors first for SOT and second for DATA 
	   and third for EOT */

    for(i = 0; i < 5 * numOfDesc; i=i+5)
    {
        if((bufferAddr & 0x00000007) != 0x0)
        {
            /* round up Address to closest allinged address */
            bufferAddr = bufferAddr + 8 - (bufferAddr & 0x00000007);  
        }


		/* Set first bisync descriptor */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST |
                                    DESC_TRANSMIT_DLE_BEFORE_BUFFER;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i].cmd_sts =  cmdStsWord | DESC_FIRST | 
									DESC_TRANSMIT_DLE_BEFORE_BUFFER;
        /* buffer size */
        txPointer[i].bytecnt = 1;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i].shadow = 1;
        /* buffer pointer */
        txPointer[i].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+1) *
                                                    DESC_SIZE);
        bufferAddr += 8;


		/* Set second bisync descriptor header data */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_BYSINC_IN_TRANSPARENT_MODE;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+1].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;
        /* buffer size */
        txPointer[i+1].bytecnt = 8;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+1].shadow = 8;
        /* buffer pointer */
        txPointer[i+1].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+1].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+2) *
                                                      DESC_SIZE);
        bufferAddr += 0x10;

		/* set third bisync descriptor STX */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
  									  DESC_TRANSMIT_DLE_BEFORE_BUFFER |
  									  DESC_BYSINC_IN_TRANSPARENT_MODE;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+2].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE | 
									  DESC_TRANSMIT_DLE_BEFORE_BUFFER;
        /* buffer size */
        txPointer[i+2].bytecnt = 1;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+2].shadow = 1;
        /* buffer pointer */
        txPointer[i+2].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+2].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + 
                                                    (i+3)*DESC_SIZE);
        bufferAddr += 8;

		/* set fourth bisync descriptor text data */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+3].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+3].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_BYSINC_IN_TRANSPARENT_MODE;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+3].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE;

        /* buffer size */
        txPointer[i+3].bytecnt = txBufferSize;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+3].shadow = txBufferSize;
        /* buffer pointer */
        txPointer[i+3].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+3].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+4) *
                                                      DESC_SIZE);
        bufferAddr += (txBufferSize + 8);

		/* set fifth bisync descriptor */
		if(bisyncMode == BISYNC_NORMAL)
			txPointer[i+4].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
								      DESC_GENERATE_ERROR_CODE_TX | DESC_LAST;

		if(bisyncMode == BISYNC_TRANSPARENT)
			txPointer[i+4].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_GENERATE_ERROR_CODE_TX | DESC_LAST |
									  DESC_TRANSMIT_DLE_BEFORE_BUFFER;

		if(bisyncMode == BISYNC_AUTO_TRANSPARENT)
			txPointer[i+4].cmd_sts =  cmdStsWord | DESC_BCE_BCC_ENABLE |
									  DESC_GENERATE_ERROR_CODE_TX | DESC_LAST |
									  DESC_TRANSMIT_DLE_BEFORE_BUFFER;
        /* buffer size */
        txPointer[i+4].bytecnt = 1;
        /* shadow size - relevant only in mpsc desc */
        txPointer[i+4].shadow = 1;
        /* buffer pointer */
        txPointer[i+4].buf_ptr = VIRTUAL_TO_PHY(bufferAddr);
        /* pointer to next descriptor */
        txPointer[i+4].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr + (i+5)
                                                      * DESC_SIZE);
        bufferAddr += 8;
    }

    if (cyclic)
    {
        /* make the descriptors chain cyclic */
        txPointer[i-1].next_desc_ptr = VIRTUAL_TO_PHY(firstTxDescAddr);
    }
    else
    {
        txPointer[i-1].next_desc_ptr = 0;
    }

    return DESC_INIT_SUCCESS;
}




