/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtMemory.c - Memory mappings and remapping functions 
*
* DESCRIPTION:
*       None.
*
* DEPENDENCIES:
*       None.
*
*******************************************************************************/

/* includes */
#include "gtMemory.h"
#include "EagleReg.h"

/*******************************************************************************
* gtMemoryGetMemoryBankBaseAddr - Returns the base address of a memory bank 
*                                 window.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       There are four memory windows (SCS[3:0]) dedicated for the DRAM in the
*       CPU's memory map. This function returns the base address of one of the 
*       DRAMs memory windows. 
*
* INPUT:
*       memoryBank - Selects one of the four memory bank windows as defined in 
*                    gtMemory.h.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit Memory window base address. If the return value is 0xffffffff 
*       the base address has no meaning!  
*
*******************************************************************************/
unsigned int gtMemoryGetMemoryBankBaseAddr(MEMORY_BANK memoryBank)
{
    unsigned int base;
    unsigned int regOffset;

    switch (memoryBank) 
    {
    case MEM_BANK0:
        regOffset = SCS_0_LOW_DECODE_ADDR;
        break;
    case MEM_BANK1:
        regOffset = SCS_1_LOW_DECODE_ADDR;
        break;
    case MEM_BANK2:
        regOffset = SCS_2_LOW_DECODE_ADDR;
        break;
    case MEM_BANK3:
        regOffset = SCS_3_LOW_DECODE_ADDR;
        break;
    default:
        return 0xffffffff;
        break;
    }
    GT_REG_READ(regOffset,&base); 
    base = base << 20;
    return base;
}

/*******************************************************************************
* gtMemoryGetPciWindowBaseAddr - Returns PCI 0/1 memory space base address.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       There are five memory windows (PCI Mem[3:0] and PCI I/O) dedicated for 
*       the PCI in the CPU's memory map.
*       This function returns the base address of one of the PCI memory 
*       windows.
*
* INPUT:
*       window - One of the CPU's PCI decode windows.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit PCI memory window base address.
*
*******************************************************************************/
unsigned int gtMemoryGetPciWindowBaseAddr(PCI_WINDOW window)
{
    unsigned int baseAddress, lowDecodeRegister;

    switch(window)
    {
    case PCI_0_IO:
        lowDecodeRegister = PCI_0_I_O_LOW_DECODE_ADDR; 
        break;
    case PCI_0_MEM0:
        lowDecodeRegister = PCI_0_MEMORY0_LOW_DECODE_ADDR; 
        break;
    case PCI_0_MEM1:
        lowDecodeRegister = PCI_0_MEMORY1_LOW_DECODE_ADDR; 
        break;
    case PCI_0_MEM2:
        lowDecodeRegister = PCI_0_MEMORY2_LOW_DECODE_ADDR;  
        break;
    case PCI_0_MEM3:
        lowDecodeRegister = PCI_0_MEMORY3_LOW_DECODE_ADDR;  
        break;
#ifdef INCLUDE_PCI_1
    case PCI_1_IO:
        lowDecodeRegister = PCI_1_I_O_LOW_DECODE_ADDR; 
        break;
    case PCI_1_MEM0:
        lowDecodeRegister = PCI_1_MEMORY0_LOW_DECODE_ADDR; 
        break;
    case PCI_1_MEM1:
        lowDecodeRegister = PCI_1_MEMORY1_LOW_DECODE_ADDR; 
        break;
    case PCI_1_MEM2:
        lowDecodeRegister = PCI_1_MEMORY2_LOW_DECODE_ADDR;  
        break;
    case PCI_1_MEM3:
        lowDecodeRegister = PCI_1_MEMORY3_LOW_DECODE_ADDR;  
        break;
#endif /* INCLUDE_PCI_1*/
    default:
        return baseAddress = 0xffffffff;
    }
    GT_REG_READ(lowDecodeRegister,&baseAddress);
    baseAddress <<= 20;
    return baseAddress;
}

/*******************************************************************************
* gtMemoryGetDeviceWindowBaseAddr - Returns the base address of a device.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       This function returns the base address of a device on the system. There 
*       are 5 possible devices (0 - 4 and one boot device) as defined in 
*       gtMemory.h. Each of the device parameters is maped to one of the CS 
*       (Devices chip selects) base address register.       
*
* INPUT:
*       device - Selects one of the five devices as defined in gtMemory.h.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit Device base address.
*
*******************************************************************************/
unsigned int gtMemoryGetDeviceWindowBaseAddr(DEVICE_WINDOW device)
{
    unsigned int base;
    unsigned int regOffset;

    switch (device) 
    {
    case DEVICE0:
        regOffset = CS_0_LOW_DECODE_ADDR;
        break;
    case DEVICE1:
        regOffset = CS_1_LOW_DECODE_ADDR;
        break;
    case DEVICE2:
        regOffset = CS_2_LOW_DECODE_ADDR;
        break;
    case DEVICE3:
        regOffset = CS_3_LOW_DECODE_ADDR;
        break;
    case BOOT_DEVICE:
        regOffset = BOOTCS_LOW_DECODE_ADDR;
        break;
    default:
        return 0xffffffff;
        break;
    }
    GT_REG_READ(regOffset, &base);
    base <<= 20;
    return base;
}

/*******************************************************************************
* gtMemoryGetMemoryBankSize - Returns the size of a memory window.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       There are four memory windows (SCS[3:0]) dedicated for the DRAM in the
*       CPU's memory map.
*       This function returns the size of one of the DRAM's memory window. 
*
* INPUT:
*       window - Selects one of the four memory windows as defined in 
*                gtMemory.h.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit memory window size or 0 for a closed or non populated window.
*
*******************************************************************************/
unsigned int gtMemoryGetMemoryBankSize(MEMORY_BANK memoryBank)
{
    unsigned int size,base;
    unsigned int highDecodeReg;
    unsigned int highValue;

    switch (memoryBank) 
    {
    case MEM_BANK0:
        highDecodeReg = SCS_0_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK1:
        highDecodeReg = SCS_1_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK2:
        highDecodeReg = SCS_2_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK3:
        highDecodeReg = SCS_3_HIGH_DECODE_ADDR;
        break;
    default:
        return 0xffffffff;
        break;
    }
    base = gtMemoryGetMemoryBankBaseAddr(memoryBank);                           
    GT_REG_READ(highDecodeReg,&highValue);
    highValue = (highValue + 1) << 20;
    if(base > highValue)
        size = 0;
    else
        size = highValue - base;
    return size;
}

/*******************************************************************************
* gtMemoryGetDeviceWindowSize - Returns the size of a device memory space.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       There are five memory windows (CS[3:0] and BOOT device) dedicated for 
*       the devices in the CPU's memory map. This function returns the size of 
*       a memory window dedicated for one of the devices.
*
* INPUT:
*       device - Selects one of the five devices as defined in gtMemory.h.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit size of a device memory space.
*
*******************************************************************************/
unsigned int gtMemoryGetDeviceWindowSize(DEVICE_WINDOW device)
{
    unsigned int size,base;
    unsigned int highAddress;
    unsigned int highValue;

    switch (device) 
    {
        case DEVICE0:
            highAddress = CS_0_HIGH_DECODE_ADDR;
            break;
        case DEVICE1:
            highAddress = CS_1_HIGH_DECODE_ADDR;
            break;
        case DEVICE2:
            highAddress = CS_2_HIGH_DECODE_ADDR;
            break;
        case DEVICE3:
            highAddress = CS_3_HIGH_DECODE_ADDR;
            break;
        case BOOT_DEVICE:
            highAddress = BOOTCS_HIGH_DECODE_ADDR;
            break;
        default:
            return 0xffffffff;
            break;
    }
    base = gtMemoryGetDeviceWindowBaseAddr(device);
    GT_REG_READ(highAddress,&highValue);
    if (highValue == 0xfff)
    {
        size = (~base) + 1;
        return size;
    }
    else
        highValue = (highValue + 1) << 20;
    if(base > highValue)
        size = 0;
    else
        size = highValue - base;
    return size;
}

/*******************************************************************************
* gtMemoryGetPciWindowSize - Returns PCI 0/1 memory window size.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.
*       This function returns one of the CPU's PCI 0/1 decode window size.
*
* INPUT:
*       window - One of the CPU's PCI decode windows.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit CPU's PCI 0/1 decode window size.
*
*******************************************************************************/
unsigned int gtMemoryGetPciWindowSize(PCI_WINDOW window)
{
    unsigned int top,base,size;
    unsigned int highDecodeRegister;

    switch(window)
    {
    case PCI_0_IO:
        highDecodeRegister = PCI_0_I_O_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM0:
        highDecodeRegister = PCI_0_MEMORY0_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM1:
        highDecodeRegister = PCI_0_MEMORY1_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM2:
        highDecodeRegister = PCI_0_MEMORY2_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM3:
        highDecodeRegister = PCI_0_MEMORY3_HIGH_DECODE_ADDR;
        break;
#ifdef INCLUDE_PCI_1
    case PCI_1_IO:
        highDecodeRegister = PCI_1_I_O_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM0:
        highDecodeRegister = PCI_1_MEMORY0_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM1:
        highDecodeRegister = PCI_1_MEMORY1_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM2:
        highDecodeRegister = PCI_1_MEMORY2_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM3:
        highDecodeRegister = PCI_1_MEMORY3_HIGH_DECODE_ADDR;
        break;
#endif /* INCLUDE_PCI_1 */
    default:
        return size = 0;
    }
    base = gtMemoryGetPciWindowBaseAddr(window);
    GT_REG_READ(highDecodeRegister,&top);
    top = (top << 20);
    if(top <= base)
        return 0;
    size = (top - base);
    size = size | 0xfffff; 
    return (size + 1);
}

/*******************************************************************************
* gtMemoryGetDeviceWidth - Returns the width of a given device.
*
* DESCRIPTION:
*       The GT's device interface supports up to 32 Bit wide devices. A device 
*       can have a  1, 2, 4 or 8 Bytes data width. This function returns the 
*       width of a device as defined by the user or the operating system.
*
* INPUT:
*       device - Selects one of the five devices as defined in gtMemory.h.
*
* OUTPUT:
*       None.
*
* RETURN:
*       Device width in Bytes (1,2,4 or 8), 0 if error had occurred.
*
*******************************************************************************/
unsigned int gtMemoryGetDeviceWidth(DEVICE_WINDOW device)
{
    unsigned int width;
    unsigned int regValue;

    GT_REG_READ(DEVICE_BANK0PARAMETERS + device*4,&regValue);
    width =  (regValue & 0x00300000) >> 20;
    switch (width)
    {
        case 0:
            return 1;
        case 1:
            return 2;
        case 2:
            return 4;
        case 3:
            return 8;
        default:
            return 0;
    }
}

/*******************************************************************************
* gtMemorySetMemoryBank - Sets new base and size for one of the CPU's memory 
*                         spaces.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window. This function sets new base and size 
*       for one of the memory banks (SCS[3:0]). It is the programmer`s 
*       responsibility to make sure that there are no conflicts with other 
*       memory spaces. When two memory spaces overlap, the MVs behavior is not
*       defined .If a bank needs to be closed, set the bankSize parameter 
*       size to 0x0.
*
* INPUT:
*       window        - One of the DRAM's memory decode windows.
*       memWindowbase - New base address for the DRAM.
*       memWindowSize - New size for SDRAM bank 0.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMemorySetMemoryBank(MEMORY_BANK memoryBank, unsigned int memWindowbase,
                           unsigned int memWindowSize)
{
    unsigned int lowRegOffset, highRegOffset, bankTopAddr;

    switch(memoryBank) 
    {
    case MEM_BANK0:
        lowRegOffset = SCS_0_LOW_DECODE_ADDR;
        highRegOffset = SCS_0_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK1:
        lowRegOffset = SCS_1_LOW_DECODE_ADDR;
        highRegOffset = SCS_1_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK2:
        lowRegOffset = SCS_2_LOW_DECODE_ADDR;
        highRegOffset = SCS_2_HIGH_DECODE_ADDR;
        break;
    case MEM_BANK3:
        lowRegOffset = SCS_3_LOW_DECODE_ADDR;
        highRegOffset = SCS_3_HIGH_DECODE_ADDR;
        break;
    default:
        return;
    }
    if(memWindowSize > 0)
    {
        bankTopAddr = (memWindowbase + memWindowSize) >> 20;
        GT_REG_WRITE(highRegOffset,bankTopAddr - 1);
        GT_REG_WRITE(lowRegOffset,memWindowbase >> 20);
    }
    else /* Close the window */
    {
        GT_REG_WRITE(highRegOffset,0x0);
    }
}

/*******************************************************************************
* gtMemorySetPciWindow - Set new base address and size for one of the CPU's 
*                           PCI decode windows.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI  Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1MB of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window. This function sets new base address and 
*       size for one of the CPU's PCI decode windows. 
*
* INPUT:
*       window        - Selects one of possible CPU's PCI memory decode windows.
*       pciWindowBase - The new base address for the PCI memory window.
*       pciWindowSize - The new size for the PCI memory window.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMemorySetPciWindow(PCI_WINDOW window, unsigned int pciWindowBase,
                          unsigned int pciWindowSize)
{
    unsigned int highDecodeValue = pciWindowBase + pciWindowSize;
    unsigned int currentLow, lowDecodeRegister, highDecodeRegister;

    switch(window)
    {
    case PCI_0_IO:
        lowDecodeRegister = PCI_0_I_O_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_0_I_O_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM0:
        lowDecodeRegister = PCI_0_MEMORY0_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_0_MEMORY0_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM1:
        lowDecodeRegister = PCI_0_MEMORY1_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_0_MEMORY1_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM2:
        lowDecodeRegister = PCI_0_MEMORY2_LOW_DECODE_ADDR;  
        highDecodeRegister = PCI_0_MEMORY2_HIGH_DECODE_ADDR;
        break;
    case PCI_0_MEM3:
        lowDecodeRegister = PCI_0_MEMORY3_LOW_DECODE_ADDR;  
        highDecodeRegister = PCI_0_MEMORY3_HIGH_DECODE_ADDR;
        break;
#ifdef INCLUDE_PCI_1
    case PCI_1_IO:
        lowDecodeRegister = PCI_1_I_O_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_1_I_O_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM0:
        lowDecodeRegister = PCI_1_MEMORY0_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_1_MEMORY0_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM1:
        lowDecodeRegister = PCI_1_MEMORY1_LOW_DECODE_ADDR; 
        highDecodeRegister = PCI_1_MEMORY1_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM2:
        lowDecodeRegister = PCI_1_MEMORY2_LOW_DECODE_ADDR;  
        highDecodeRegister = PCI_1_MEMORY2_HIGH_DECODE_ADDR;
        break;
    case PCI_1_MEM3:
        lowDecodeRegister = PCI_1_MEMORY3_LOW_DECODE_ADDR;  
        highDecodeRegister = PCI_1_MEMORY3_HIGH_DECODE_ADDR;
        break;
#endif /* INCLUDE_PCI_1 */
    default:                     
        return;
    }
    GT_REG_READ(lowDecodeRegister,&currentLow);
    pciWindowBase = pciWindowBase >> 20;
    pciWindowBase |= (currentLow & 0x07000000);
    highDecodeValue = highDecodeValue >> 20;   
    if(pciWindowSize > 0)
    {
        GT_REG_WRITE(highDecodeRegister,highDecodeValue - 1);
        GT_REG_WRITE(lowDecodeRegister,pciWindowBase);
        return;
    }
    /* If we got so far, the size is 0. */
    GT_REG_WRITE(highDecodeRegister,0x0);
}

/*******************************************************************************
* gtMemorySetDeviceWindow - Sets new bases and size for a device memory window.
*
* DESCRIPTION:
*       The CPU interface address decoding map consists of 21 address windows 
*       for the different devices (SCS[3:0] ,PCI Mem 0/1/2/3 etc.). Each window
*       can have a minimum of 1Mbytes of address space, and up to 4Gbyte space.  
*       Each address window is defined by two registers - Low and High. The CPU 
*       address is compared with the values in the various CPU Low and High 
*       decode registers of each window until a match is found and the address 
*       is than targeted to that window.This function sets new base and size for 
*       device 0 memory space(CS0). It is the programmer`s responsibility to 
*       make sure that there are no conflicts with other memory spaces. When two
*       memory spaces overlap, the GTs behavior is not defined .If the memory 
*       space needs to be closed, set the 'device0Size' parameter size to 0x0, 
*       it gives the low decode address register max value (0xffff) and to the 
*       high decode address register min value (0x0).
*
* INPUT:
*       device0Base - new base address for device 0 memory space.
*       device0Size - new size for device 0 memory space.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMemorySetDeviceWindow(DEVICE_WINDOW device, unsigned int deviceBase,
                             unsigned int deviceSize) 
{
    unsigned int deviceTop;
    unsigned int highRegOffset, lowRegOffset;
    
    switch (device) 
    {
        case DEVICE0:
            lowRegOffset = CS_0_LOW_DECODE_ADDR;
            highRegOffset = CS_0_HIGH_DECODE_ADDR;
            break;
        case DEVICE1:
            lowRegOffset = CS_1_LOW_DECODE_ADDR;
            highRegOffset = CS_1_HIGH_DECODE_ADDR;
            break;
        case DEVICE2:
            lowRegOffset = CS_2_LOW_DECODE_ADDR;
            highRegOffset = CS_2_HIGH_DECODE_ADDR;
            break;
        case DEVICE3:
            lowRegOffset = CS_3_LOW_DECODE_ADDR;
            highRegOffset = CS_3_HIGH_DECODE_ADDR;
            break;
        case BOOT_DEVICE:
            lowRegOffset = BOOTCS_LOW_DECODE_ADDR;
            highRegOffset = BOOTCS_HIGH_DECODE_ADDR;
            break;
        default:
            return;
            break;
    } 
    if(deviceSize > 0)
    {
        deviceTop = (deviceBase + deviceSize) >> 20;
        GT_REG_WRITE(highRegOffset,deviceTop - 1);
        GT_REG_WRITE(lowRegOffset,deviceBase >> 20);
    }
    else
    {
        GT_REG_WRITE(highRegOffset,0x0);
    }
}

/*******************************************************************************
* gtMemoryMapInternalRegistersSpace - Sets new base address for the internal 
*                                     registers memory space.
*
* DESCRIPTION:
*       This function set new base address for the internal registers memory
*       space (the size is fixed and cannot be modified). The function does not 
*       handle overlapping with other memory spaces, it is the programer's 
*       responsibility to ensure that overlapping does not occur. 
*       When two memory spaces overlap, the GTs behavior is not defined.
*
* INPUT:
*       internalRegBase - new base address for the internal registers memory 
*                         space.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
void gtMemoryMapInternalRegistersSpace(unsigned int internalRegBase)
{
    unsigned int currentValue;
    unsigned int internalValue = internalRegBase;

    internalRegBase = (internalRegBase >> 20);
    GT_REG_READ(INTERNAL_SPACE_DECODE,&currentValue);
    internalRegBase = (currentValue & 0xffff0000) | internalRegBase;
    GT_REG_WRITE(INTERNAL_SPACE_DECODE,internalRegBase);
    gtInternalRegBaseAddr = internalValue;
}

/*******************************************************************************
* gtMemoryGetInternalRegistersSpace - Returns the internal registers Base 
*                                     address.
*
* DESCRIPTION:
*       This function returns the base address of  the internal registers 
*       memory space .
*
* INPUT:
*       None.
*
* OUTPUT:
*       None.
*
* RETURN:
*       32 bit base address of the internal registers memory space.
*
*******************************************************************************/
unsigned int gtMemoryGetInternalRegistersSpace(void)
{
    return gtInternalRegBaseAddr;
}

/*******************************************************************************
* gtMemorySetProtectRegion - Set protection mode for one of the 8 regions.
*
* DESCRIPTION:
*       The CPU interface supports configurable access protection. This includes
*       up to eight address ranges defined to a different protection type : 
*       whether the address range is cacheable or not, whether it is writable or
*       not , and whether it is accessible or not. A Low and High registers 
*       define each window while the minimum address range of each window is 
*       1Mbyte. An address driven by the CPU, in addition to the address 
*       decoding and remapping process, is compared against the eight Access 
*       Protection Low/High registers , if an address matches one of the windows
*       , the GT device checks the transaction type against the protection bits 
*       defined in CPU Access Protection register, to determine if the access is
*       allowed. This function set a protection mode to one of the 8 possible 
*       regions.
*      NOTE:
*       The CPU address windows are restricted to a size of  2 power n and the 
*       start address must be aligned to the window size. For example, if using
*       a 16 MB window, the start address bits [23:0] must be 0.The GT's 
*       internal registers space is not protected, even if the access protection
*       windows contain this space.
*
* INPUT:
*       region - selects which region to be configured. The values defined in 
*                gtMemory.h:
*
*                 - MEM_REGION0
*                 - MEM_REGION1
*                 - etc.
*
*       memAccess - Allows or forbids access (read or write ) to the region. The
*                   values defined in gtMemory.h:
*
*                    - MEM_ACCESS_ALLOWED 
*                    - MEM_ACCESS_FORBIDEN
*
*       memWrite - CPU write protection to the region. The values defined in 
*                  gtMemory.h: 
*
*                   - MEM_WRITE_ALLOWED
*                   - MEM_WRITE_FORBIDEN
*
*       cacheProtection - Defines whether caching the region is allowed or not. 
*                         The values defined in gtMemory.h:
*
*                          - MEM_CACHE_ALLOWED
*                          - MEM_CACHE_FORBIDEN
*
*       baseAddress - Determines the regions base Address.
*       regionSize - Determines the regions size.
*
* OUTPUT:
*       None.
*
* RETURN:
*       None.
*
*******************************************************************************/
bool gtMemorySetProtectRegion(MEMORY_PROTECT_REGION region,
                              MEMORY_ACCESS memAccess,
                              MEMORY_ACCESS_WRITE memWrite,
                              MEMORY_CACHE_PROTECT cacheProtection,
                              unsigned int baseAddress, 
                              unsigned int regionSize)
{
    unsigned int protectHigh = baseAddress + regionSize;

    if(regionSize == 0) /* closing the region */
    {
        GT_REG_WRITE(CPU_LOW_PROTECT_ADDR_0 + 0x10*region,0x0000ffff);
        GT_REG_WRITE(CPU_HIGH_PROTECT_ADDR_0 + 0x10*region,0);
        return true;
    }
    baseAddress =  (baseAddress & 0xfff00000) >> 20;
    baseAddress = baseAddress | memAccess << 16 |  memWrite << 17
                     | cacheProtection << 18;
    GT_REG_WRITE(CPU_LOW_PROTECT_ADDR_0 + 0x10*region,baseAddress);
    protectHigh = (protectHigh & 0xfff00000) >> 20;
    GT_REG_WRITE(CPU_HIGH_PROTECT_ADDR_0 + 0x10*region,protectHigh - 1);
    return true;
}

/*******************************************************************************
* gtMemorySetPciRemapValue - Set a remap value to a PCI memory space target.
*
* DESCRIPTION:
*       In addition to the address decoding mechanism, the CPU has an address 
*       remapping mechanism to be used by every PCI decoding window. Each PCI 
*       window can be remaped to a desired address target according to the remap
*       value within the remap register. The address remapping is useful when a 
*       CPU address range must be reallocated to a different location on the 
*       PCI bus. Also, it enables CPU access to a PCI agent located above the 
*       4Gbyte space. On system boot, each of the PCI memory spaces is maped to 
*       a defualt value (see CPU interface section in the GT spec for the 
*       default values). The remap mechanism does not always produce the deisred
*       address on the PCI bus because of the remap mechanism way of working 
*       (to fully understand why, please see the 'Address Remapping' section in 
*       the GT's spec). Therefor, this function sets a desired remap value to 
*       one of the PCI memory windows and return the effective address that 
*       should be used when exiting the PCI memory window. You should ALWAYS use
*       the returned value by this function when remapping a PCI window and 
*       exiting it. If for example the base address of PCI0 memory 0 is 
*       0x90000000, the top address is 0x97ffffff and the remap value is 
*       0x14000000, the function will return the value of 0x94000000 that MUST
*       be used to exit this memory window in order to achive the deisred 
*       remapping. Note that in some cases the fnuction will return the same 
*       value as the window base address.
*        
* INPUT:
*       memoryWindow   - One of the PCI memory windows as defined in gtMemory.h
*       remapValueLow  - The low remap value.
*       remapValueHigh - The high remap value.
* OUTPUT:
*       None.
*
* RETURN:
*       The effective base address to exit the PCI, or 0xffffffff if one of the
*       parameters is erroneous, the effective base address is higher the top
*       decode value or the window is closed.
*
*******************************************************************************/
unsigned int gtMemorySetPciRemapValue(PCI_WINDOW memoryWindow,
                                      unsigned int remapValueHigh,
                                      unsigned int remapValueLow)
{
    int          remapValueFirstOneIndex = 0;
    unsigned int pciMemWindowLowDecodeReg = 0,lowDecodeValue = 0;
    unsigned int pciMemWindowHighDecodeReg = 0,highDecodeValue = 0;
    unsigned int shift,xorValue = 0,choppedPartMask = 0;
    unsigned int effectiveBaseAddress,remapRegLow,remapRegHigh;
    unsigned int tempRemapValue;

    /* Initializing the low and high decode variables of the PCI 
       memory windows */
    switch(memoryWindow)
    {
    case PCI_0_IO:
        pciMemWindowLowDecodeReg = PCI_0_I_O_LOW_DECODE_ADDR; 
        pciMemWindowHighDecodeReg = PCI_0_I_O_HIGH_DECODE_ADDR;
        remapRegLow = PCI_0_I_O_ADDR_REMAP;
        remapRegHigh = PCI_0_I_O_ADDR_REMAP;
        break;
    case PCI_0_MEM0:
        pciMemWindowLowDecodeReg = PCI_0_MEMORY0_LOW_DECODE_ADDR; 
        pciMemWindowHighDecodeReg = PCI_0_MEMORY0_HIGH_DECODE_ADDR;
        remapRegLow = PCI_0_MEMORY0_ADDR_REMAP;
        remapRegHigh = PCI_0_MEMORY0_HIGH_ADDR_REMAP;
        break;
    case PCI_0_MEM1:
        pciMemWindowLowDecodeReg = PCI_0_MEMORY1_LOW_DECODE_ADDR; 
        pciMemWindowHighDecodeReg = PCI_0_MEMORY1_HIGH_DECODE_ADDR;
        remapRegLow = PCI_0_MEMORY1_ADDR_REMAP;
        remapRegHigh = PCI_0_MEMORY1_HIGH_ADDR_REMAP;
        break;
    case PCI_0_MEM2:
        pciMemWindowLowDecodeReg = PCI_0_MEMORY2_LOW_DECODE_ADDR;  
        pciMemWindowHighDecodeReg = PCI_0_MEMORY2_HIGH_DECODE_ADDR;
        remapRegLow = PCI_0_MEMORY2_ADDR_REMAP;
        remapRegHigh = PCI_0_MEMORY2_HIGH_ADDR_REMAP;
        break;
    case PCI_0_MEM3:
        pciMemWindowLowDecodeReg = PCI_0_MEMORY3_LOW_DECODE_ADDR;  
        pciMemWindowHighDecodeReg = PCI_0_MEMORY3_HIGH_DECODE_ADDR;
        remapRegLow = PCI_0_MEMORY3_ADDR_REMAP;
        remapRegHigh = PCI_0_MEMORY3_HIGH_ADDR_REMAP;
        break;
#ifdef INCLUDE_PCI_1
    case PCI_1_IO:
        pciMemWindowLowDecodeReg = PCI_1_I_O_LOW_DECODE_ADDR;  
        pciMemWindowHighDecodeReg = PCI_1_I_O_HIGH_DECODE_ADDR;
        remapRegLow = PCI_1_I_O_ADDR_REMAP;
        remapRegHigh = PCI_1_I_O_ADDR_REMAP;
        break;
    case PCI_1_MEM0:
        pciMemWindowLowDecodeReg = PCI_1_MEMORY0_LOW_DECODE_ADDR; 
        pciMemWindowHighDecodeReg = PCI_1_MEMORY0_HIGH_DECODE_ADDR;
        remapRegLow = PCI_1_MEMORY0_ADDR_REMAP;
        remapRegHigh = PCI_1_MEMORY0_HIGH_ADDR_REMAP;
        break;
    case PCI_1_MEM1:
        pciMemWindowLowDecodeReg = PCI_1_MEMORY1_LOW_DECODE_ADDR; 
        pciMemWindowHighDecodeReg = PCI_1_MEMORY1_HIGH_DECODE_ADDR;
        remapRegLow = PCI_1_MEMORY1_ADDR_REMAP;
        remapRegHigh = PCI_1_MEMORY1_HIGH_ADDR_REMAP;
        break;
    case PCI_1_MEM2:
        pciMemWindowLowDecodeReg = PCI_1_MEMORY2_LOW_DECODE_ADDR;  
        pciMemWindowHighDecodeReg = PCI_1_MEMORY2_HIGH_DECODE_ADDR;
        remapRegLow = PCI_1_MEMORY2_ADDR_REMAP;
        remapRegHigh = PCI_1_MEMORY2_HIGH_ADDR_REMAP;
        break;
    case PCI_1_MEM3:
        pciMemWindowLowDecodeReg = PCI_1_MEMORY3_LOW_DECODE_ADDR;  
        pciMemWindowHighDecodeReg = PCI_1_MEMORY3_HIGH_DECODE_ADDR;
        remapRegLow = PCI_1_MEMORY3_ADDR_REMAP;
        remapRegHigh = PCI_1_MEMORY3_HIGH_ADDR_REMAP;
        break;
#endif /* INCLUDE_PCI_1 */
    default:
        /* Retrun an invalid effective base address */
        return effectiveBaseAddress = 0xffffffff;
    }
    /* Writing the remap value to the remap regisers */
    GT_REG_WRITE(remapRegHigh,remapValueHigh);
    GT_REG_WRITE(remapRegLow,remapValueLow >> 20);
    /* Reading the base address of the given PCI memory window and holding
       it temporarily in 'effectiveBaseAddress' */
    effectiveBaseAddress = (GTREGREAD(pciMemWindowLowDecodeReg) << 20);
    /* Reading the values from the decode registers */
    lowDecodeValue = GTREGREAD(pciMemWindowLowDecodeReg) & 0xfff;
    highDecodeValue = GTREGREAD(pciMemWindowHighDecodeReg) & 0xfff;
    /* Find out where is the first 'one' of the remap value
       (bit possition that equals to '1') from right to left */
    tempRemapValue = remapValueLow >> 20;
    while(((tempRemapValue & BIT0) == 0) && (remapValueFirstOneIndex < 12))
    {
        /* Incrementing the index */
        remapValueFirstOneIndex++;
        /* Moving to the next bit on the left */
        tempRemapValue >>= BIT0;
    }
    /* If the last 'one' of the remap value is within the range of the matching 
       bits from MSB to LSB of the low and high remap values, the remaping will 
       be 'as is' without any special masking/manipulating/changes */
    xorValue = lowDecodeValue ^ highDecodeValue;
    if(xorValue < ((unsigned int)(BIT0 << remapValueFirstOneIndex)) ||
                                                /* Remaping to 0 */
                                                (remapValueFirstOneIndex == 12))
    {
        /* Return the effective base address without any change */
        return effectiveBaseAddress;
    }
    /* If we got so far, it means that when an address comming from the CPU 
       goe's through the remap mechanism, a part of the remap value (that 
       should be merged with the address) is chopped out and the effective 
       address on the PCI bus is NOT the address as we were expecting it to be.
       The chopped part is the delta between the first one of the remap value 
       (from right to left) and the first mismatch of the corresponding 
       bits in the high and low decode values (from left to right). We need
       add this chopped part to the address - resulting the real effective 
       address. */
    /* Counting the number of matching bits from MSB to LSB of the low and high
       decode values */
    shift = 0;
    while((((xorValue << shift) & BIT11) == 0) && (shift < 12))
        shift++; /* == matching bits */
    if(shift == 12)
    {                             
        /* Return an invalid effective base address since the PCI memory
           window is closed */
        return effectiveBaseAddress = 0xffffffff;
    }
    /* Creating the mask to the chopped part */
    shift++;
    tempRemapValue = remapValueLow >> 20;
    while(shift < 13)
        choppedPartMask |= (0x1 << (12 - shift++));
    /* Setting the real effective base address to be used when going out to the
       PCI bus through decode windows */
    effectiveBaseAddress |= ((tempRemapValue & choppedPartMask) << 20);
    /* The PCI window cannot be remaped to this remap value since the 
       effective base address */
    if(effectiveBaseAddress > (highDecodeValue << 20))
        return effectiveBaseAddress = 0xffffffff;
    return effectiveBaseAddress;
}
/*******************************************************************************
* gtMemoryGetDeviceParam - Extract the device parameters from the device bank
*                          parameters register.
*
* DESCRIPTION:
*       To allow interfacing with very slow devices and fast synchronous SRAMs,
*       each device can be programed to different timing parameters. Each bank 
*       has its own parameters register. Bank width can be programmed to 8, 16,
*       or 32-bits. Bank timing parameters can be programmed to support 
*       different device types (e.g. Sync Burst SRAM, Flash , ROM, I/O 
*       Controllers). The GT allows you to set timing parameters and width for 
*       each device through parameters register .
*       This function extracts the parameters described from the Device Bank 
*       parameters register and fills the given 'deviceParam' (defined in 
*       gtMemory.h) structure with the read data.
*
* INPUT:
*       deviceParam -  pointer to a structure GT_DEVICE_PARAM (defined in 
*                      gtMemory.h).For details about each structure field please
*                      see the device timing parameter section in the GT 
*                      datasheet.
*       device      -   Select on of the five device banks (defined in 
*                      gtMemory.h) :
*         
*                       - DEVICE0
*                       - DEVICE1
*                       - DEVICE2
*                       - etc.
*
* OUTPUT:
*       None.
*
* RETURN:
*       false if one of the parameters is erroneous,true otherwise.
*
*******************************************************************************/
bool gtMemoryGetDeviceParam(DEVICE_PARAM *deviceParam, DEVICE_WINDOW device)
{
    unsigned int valueOfReg;
    unsigned int calcData;

    GT_REG_READ(DEVICE_BANK0PARAMETERS + 4 * device, &valueOfReg);
    calcData = (0x7 & valueOfReg) + ((0x400000 & valueOfReg) >> 19);
    deviceParam -> turnOff = calcData;          /* Turn Off */

    calcData = ((0x78 & valueOfReg) >> 3) + ((0x800000 & valueOfReg) >> 19);
    deviceParam -> acc2First = calcData;        /* Access To First */

    calcData = ((0x780 & valueOfReg) >> 7) + ((0x1000000 & valueOfReg) >> 20);
    deviceParam -> acc2Next = calcData;         /* Access To Next */
                                                
    calcData = ((0x3800 & valueOfReg) >> 11) + ((0x2000000 & valueOfReg) >> 22);
    deviceParam -> ale2Wr = calcData;           /* Ale To Write */
                                                
    calcData = ((0x1c000 & valueOfReg) >> 14) + 
                                               ((0x4000000 & valueOfReg) >> 23);
    deviceParam -> wrLow = calcData;            /* Write Active */

    calcData = ((0xe0000 & valueOfReg) >> 17) + 
                                               ((0x8000000 & valueOfReg) >> 24);
    deviceParam -> wrHigh = calcData;           /* Write High */

    calcData = ((0x300000 & valueOfReg) >> 20);
    switch (calcData) 
    {       
    case 0:
        deviceParam -> deviceWidth = 1;         /* one Byte - 8-bit */
        break;
    case 1:
        deviceParam -> deviceWidth = 2;         /* two Bytes - 16-bit */
        break;
    case 2:
        deviceParam -> deviceWidth = 4;         /* four Bytes - 32-bit */
        break;
    case 3:
        deviceParam -> deviceWidth = 8;         /* eight Bytes - 64-bit */
        break;
    default:
        deviceParam -> deviceWidth = 1;
        break;
    }
    return true;
}

/*******************************************************************************
* gtMemorySetDeviceParam - Set new parameters for a device.
*
*
* DESCRIPTION:
*       To allow interfacing with very slow devices and fast synchronous SRAMs,
*       each device can be programed to different timing parameters. Each bank 
*       has its own parameters register. Bank width can be programmed to 8, 16,
*       or 32-bits. Bank timing parameters can be programmed to support 
*       different device types (e.g. Sync Burst SRAM, Flash , ROM, I/O 
*       Controllers). The GT allows you to set timing parameters and width for 
*       each device through parameters register. This function set new 
*       parameters to a device Bank from the delivered structure 'deviceParam' 
*       (defined in gtMemory.h). The structure must be initialized with data 
*       prior to the use of these function.
*
* INPUT:
*       deviceParam -  pointer to a structure GT_DEVICE_PARAM (defined in 
*                      gtMemory.h).For details about each structure field please
*                      see the device timing parameter section in the GT 
*                      datasheet.
*       deviceNum  -   Select on of the five device banks (defined in 
*                      gtMemory.h) :
*         
*                       - DEVICE0
*                       - DEVICE1
*                       - DEVICE2
*                       - etc.
*
* OUTPUT:
*       None.
*
* RETURN:
*       false if one of the parameters is erroneous,true otherwise.
*
*******************************************************************************/
bool gtMemorySetDeviceParam(DEVICE_PARAM *deviceParam, DEVICE_WINDOW device)
{
    unsigned int valueForReg;

    if((deviceParam -> turnOff >= 0xf) || (deviceParam -> acc2First >= 0x1f) || 
       (deviceParam -> acc2Next >= 0x1f) || (deviceParam -> ale2Wr >= 0xf) ||
        (deviceParam -> wrLow >= 0xf) || (deviceParam -> wrHigh >= 0xf))
        return false;
    valueForReg = (((deviceParam -> turnOff) & 0x7) |
                   (((deviceParam -> turnOff) & 0x8) << 19) |
                   (((deviceParam -> acc2First) & 0xf) << 3) |
                   (((deviceParam -> acc2First) & 0x10) << 19) |
                   (((deviceParam -> acc2Next) & 0xf) << 7) |
                   (((deviceParam -> acc2Next) & 0x10) << 20) |
                   (((deviceParam -> ale2Wr) & 0x7) << 11) |
                   (((deviceParam -> ale2Wr) & 0xf) << 22) |
                   (((deviceParam -> wrLow) & 0x7) << 14) |
                   (((deviceParam -> wrLow) & 0xf) << 23) |
                   (((deviceParam -> wrHigh) & 0x7) << 17) |
                   (((deviceParam -> wrHigh) & 0xf) << 24));
    /* insert the device width: */
    switch(deviceParam->deviceWidth)
    {
    case 1:
        valueForReg = valueForReg | _8BIT;
        break;
    case 2:
        valueForReg = valueForReg | _16BIT;
        break;
    case 4:
        valueForReg = valueForReg | _32BIT;
        break;
    case 8:
        valueForReg = valueForReg | _64BIT;
        break;
    default:
        valueForReg = valueForReg | _8BIT;
        break;
    }
    GT_REG_WRITE(DEVICE_BANK0PARAMETERS + 4 * device, valueForReg);
    return true;
}

#ifdef INCLUDE_SNOOP_SUPPORT
/*******************************************************************************
* gtMemorySetRegionSnoopMode - This function configure one of the 4 regions 
*                              which supports Cache Coherency (for PPC only!!!).
*
* DESCRIPTION:
*       The GT supports full cache coherency between the CPU L1 cache, L2 cache
*       and SDRAM. A Cache coherent region will maintain the data in that region
*       coherent with the data in cache, this is done by the snoop mechanism. 
*       Any access to the SDRAM (PCI, IDMA, and comm ports) may result in a 
*       snoop transaction driven by the GT on the CPU bus (in case that 
*       approach-address hits one of the regions) if the region is configured in
*       snoop mode.
*
* INPUT:
*       region - One of the four possible regions to be configured as cache 
*                coherent (defined in gtMemory.h).
*       snoopType - Select one of the four snoop option (defined in gtMemory.h):
*                    
*                    - MEM_NO_SNOOP 
*                    - MEM_SNOOP_WT
*                    - etc.
* 
*       baseAddress - Sets the region's base address.
*       regionSize - Sets the region's size 
*       
* OUTPUT:
*       None.
*
* RETURN:
*       false if one of the parameters is erroneous,true otherwise.
*
*******************************************************************************/
bool gtMemorySetRegionSnoopMode(MEMORY_SNOOP_REGION region,
                                MEMORY_SNOOP_TYPE snoopType,
                                unsigned int baseAddress,
                                unsigned int regionSize)
{
    unsigned int snoopXbaseAddress;
    unsigned int snoopXtopAddress;
    unsigned int data;
    unsigned int snoopHigh = baseAddress + regionSize;

    if( (region > MEM_SNOOP_REGION3) || (snoopType > MEM_SNOOP_WB) )
        return false;
    snoopXbaseAddress = SNOOP_BASE_ADDR_0 + 0x10 * region;
    snoopXtopAddress = SNOOP_TOP_ADDR_0 + 0x10 * region;
    if(regionSize == 0) /* closing the region */
    {
        GT_REG_WRITE(snoopXbaseAddress,0x0000ffff);
        GT_REG_WRITE(snoopXtopAddress,0);
        return true;
    }
    baseAddress = baseAddress & 0xfff00000;
    data = (baseAddress >> 20) | snoopType << 16;
    GT_REG_WRITE(snoopXbaseAddress,data);
    snoopHigh = (snoopHigh & 0xfff00000) >> 20;
    GT_REG_WRITE(snoopXtopAddress,snoopHigh - 1);
    return true;
}
#endif /* INCLUDE_SNOOP_SUPPORT */
