/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/setup_uart.c,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    Low-level helper code to setup the uart for data transmission.
 *
 * Public Procedures:
 *    getchar                 Read a char from the 16550 UART block. 
 *                            When one is avail.
 *    putchar                 Write a u8 to the 16550.
 *    setup_uart              Prepare the 16550 for 38400 NP 1stop also
 *                            set the pin mux to UART from USB
 *
 * Private Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

#include "defs.h"
#include "access.h"
#include "eagle.h"
#include "setup_uart.h"
#include "xmodem.h"
#include "test.h"

//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================

/******************************************************************************
 *
 * Name: getchar
 *
 * Description:
 *   Return the next char available on UART receive. If no char is
 *   ready, it will wait. Should no char still come after a timeout
 *   passes, then return back to the   uart_boot() code and NAK the
 *   packet. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *   Must be run with cache off or NAK_TIMEOUT loop will not be 10sec.
 *   Requires ARM ADS1.2 Thumb compiler. Again, timeout loop sensitivity.
 *
 * Arguments:
 *   Arg1( u16 ): 16 bits of data to appear at the pins.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern s32
getchar( u8 *pu8c )
{
    s32 s32counter = 0;
retry:
    s32counter++;
    if((s32counter>>16) >= NAK_TIMEOUT_VAL) {
	return FAIL;
    }
    if(!(REG32(LINE_STATUS) & DATA_READY)) {
        goto retry;
    }
    *pu8c = REG32(Rx_BUFFER_REG);
    return OK;
}
/******************************************************************************
 *
 * Name: putchar
 *
 * Description:
 *   Put a char into the UART transmit buffer. Wait until that buffer
 *   is ready. No timeout here, assume we can always be ready to send
 *   after a wait. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *
 * Arguments:
 *   Arg1( u8 ): 8 bits of data to be placed in the 16550 xmit buffer.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void
putchar( u8 u8c )
{
    while(!(REG32(LINE_STATUS) & Tx_EMPTY));    // Tx_EMPTY is 1 when 
	                                        // empty
    REG16(Tx_HOLDING_REG) = u8c;
}

/******************************************************************************
 *
 * Name: setup_uart
 *
 * Description:
 *   Place the 16550 into a useable state for us. This means the the
 *   baud rate needs to be 38400, no parity, one stop bit. Also the
 *   pad mux needs to get flipped from the default of USB to UART. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void 
setup_uart( void )
{
    u32 u32tmp;
    /* Mode based register file in the synopsys IP for the 16550. Need
       to write a special pattern to LINE_CONTROL and then the low
       registers in the file become config registers. Finally write
       the EIGHT_BIT request to the LINE_CONTROL and also deassert
       DLAB in doing so, this reverts the low register back to their
       RX&TX functions */
    REG32(SI_DIV)    = 2;		     // how 8mhz is derived
                                             // from 44mhz. No doc on
                                             // this apparantly...
    REG32(LINE_CONTROL)  = DLAB;
    REG32(DIVISOR_LOW)   = BAUDRATE_DIV;
    REG32(LINE_CONTROL)  = EIGHT_BIT;
    /* Get the value from the pad ctrl reg. so we can selectively
       disable the USB pin */
    u32tmp               = REG32(PI_PAD_CTRL);
    /* De-assert bit11 the mux for USB/UART --> UART (according to the
       verilog) */  
    REG32(PI_PAD_CTRL)   = u32tmp & (~bit(11)); 
}
