/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: X:/APCVSDEPOT/Bootloader/AP21/ap21_tftp_boot/uart.c,v 1.1.1.1 2002/12/30 20:44:05 cvu Exp $
 *
 * Purpose:
 *    Low-level helper code to setup the uart for data transmission.
 *
 * Public Procedures:
 *    get_char                 Read a char from the 16550 UART block. 
 *                            When one is avail.
 *    put_char                 Write a u8 to the 16550.
 *    setup_uart              Prepare the 16550 for 38400 NP 1stop also
 *                            set the pin mux to UART from USB
 *
 * Private Procedures:
 *    None.
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

#include "88E6218.h"
#include "uart.h"

//=============================================================================
//                         CODED PUBLIC PROCEDURES
//=============================================================================

/******************************************************************************
 *
 * Name: get_char
 *
 * Description:
 *   Return the next char available on UART receive. If no char is
 *   ready, it will wait. Should no char still come after a timeout
 *   passes, then return back to the   uart_boot() code and NAK the
 *   packet. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *   Must be run with cache off or NAK_TIMEOUT loop will not be 10sec.
 *   Requires ARM ADS1.2 Thumb compiler. Again, timeout loop sensitivity.
 *
 * Arguments:
 *   Arg1( u16 ): 16 bits of data to appear at the pins.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/


int get_char( void ) {

    int s32counter = 0;
retry:
    s32counter++;
    if((s32counter>>16) >= NAK_TIMEOUT_VAL) {
	return FAIL;
    }
    if(!(GT_READWORD(LINE_STATUS) & DATA_READY)) {
        goto retry;
    }
    return GT_READWORD(Rx_BUFFER_REG);
}


/******************************************************************************
 *
 * Name: put_char
 *
 * Description:
 *   Put a char into the UART transmit buffer. Wait until that buffer
 *   is ready. No timeout here, assume we can always be ready to send
 *   after a wait. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *
 * Arguments:
 *   Arg1( u8 ): 8 bits of data to be placed in the 16550 xmit buffer.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/

int put_char(int u8c)
{
	unsigned int stat;
	GT_READ_WORD(LINE_STATUS, &stat);
    while(!(stat & Tx_EMPTY)) {    // Tx_EMPTY is 1 when empty
    	GT_READ_WORD(LINE_STATUS, &stat);
    }
    GT_WRITE_SHORT(Tx_HOLDING_REG,u8c);
    return (u8c);
}

void outbyte(int c)
{
    put_char(c);
}


int inbyte(void)
{
    int c;

    c = get_char();
    put_char(c);
    return (c);
}



/******************************************************************************
 *
 * Name: setup_uart
 *
 * Description:
 *   Place the 16550 into a useable state for us. This means the the
 *   baud rate needs to be 38400, no parity, one stop bit. Also the
 *   pad mux needs to get flipped from the default of USB to UART. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void 
setup_uart( void )
{
    UINT32 u32tmp;

//    GT_WRITE_WORD(0x8000c000, 1); // preset

    /* Mode based register file in the synopsys IP for the 16550. Need
       to write a special pattern to LINE_CONTROL and then the low
       registers in the file become config registers. Finally write
       the EIGHT_BIT request to the LINE_CONTROL and also deassert
       DLAB in doing so, this reverts the low register back to their
       RX&TX functions */
    GT_WRITE_WORD(SI_DIV,2);		     // how 8mhz is derived
                                             // from 44mhz. No doc on
                                             // this apparantly...
    GT_WRITE_WORD(LINE_CONTROL, DLAB);
    GT_WRITE_WORD(DIVISOR_LOW,BAUDRATE_DIV);
    GT_WRITE_WORD(LINE_CONTROL,EIGHT_BIT);
    /* Get the value from the pad ctrl reg. so we can selectively
       disable the USB pin */
   /* eran - need to check if needed */
   
   u32tmp               = GT_READWORD(PI_PAD_CTRL);
    /* De-assert bit11 the mux for USB/UART --> UART (according to the
       verilog) */  
   GT_WRITE_WORD(PI_PAD_CTRL, (u32tmp & (~bit(11))));
}
