/*
*                Copyright 2004, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    dsport.c
 * \brief   Routines for distribution service port used for Live AP.
 */

/*=============================================================================
 *                               INCLUDE FILES
 *=============================================================================
 */
#include "ap_api.h"
#include "dsport.h"
#include "timer.h"
#include "wl_macros.h"

#ifdef AP_PCI
/*=============================================================================
 *                                DEFINITIONS                                  
 *=============================================================================
 */
#define DS_PORT_ETH 1
#define DS_PORT_PCI 0

#define DS_GPIO_POWER_SENSE_BIT    0x06
#define DS_GPIO_ETHER_CONTROL_BIT  0x08
#define DS_GPIO_ETHER_ENABLE       0x00
#define DS_GPIO_ETHER_DISABLE      0x01

/*=============================================================================
 *                   PRIVATE PROCEDURES (ANSI Prototypes)
 *=============================================================================
 */
static void dsPort_ControlEthernetDisable_PciEnable(void);
static void dsPort_ControlEthernetEnsable_PciDisable(void);

/*=============================================================================
 *                          MODULE LEVEL VARIABLES
 *=============================================================================
 */
#ifdef AP_CARD
static UINT32 dsPortEth = DS_PORT_PCI; /* Default port on startup is PCI */
UINT32 pci_power = 1;
#else
static UINT32 dsPortEth = DS_PORT_ETH; /* Default port on startup is PCI */
UINT32 pci_power = 0;
#endif
/*=============================================================================
 *                         CODED PUBLIC PROCEDURES
 *=============================================================================
 */
/******************************************************************************
 *
 * Name: dsPort_Init
 *
 * Description:
 *   This routine is to set the dsport to use the Ethernet port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern void dsPort_Init(void)
{
    if(msi_wl_ReadGPIO(DS_GPIO_POWER_SENSE_BIT))
    {
        dsPort_SetPci();
    }else
    {
        dsPort_SetEth(0);
    }
}
/******************************************************************************
 *
 * Name: dsPort_SetEth
 *
 * Description:
 *   This routine is to set the dsport to use the Ethernet port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT8 ether_port_down;
extern UINT32 dsPort_SetEth(int flag)
{
#ifndef AP_CARD
    ether_port_down = 0;
    set_pci_data_not_ready();
    dsPortEth = DS_PORT_ETH;
    dsPort_ControlEthernetEnable_PciDisable();
    if (flag) {
        WL_WRITE_REGS32(0x80000410, 0xd0000000);
    }
#endif    
    return 0;
}

/******************************************************************************
 *
 * Name: dsPort_SetPci
 *
 * Description:
 *   This routine is to set the dsport to use the PCI port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_SetPci(void)
{
#ifndef AP_CARD
     ether_port_down = 1;
    dsPort_ControlEthernetDisable_PciEnable();
#endif    
    return 0;
}
extern UINT32 dsPort_SetPciSw(void)
{
#ifndef AP_CARD
    dsPortEth = DS_PORT_PCI;
#endif    
    return 0;
}
extern BOOLEAN dsPort_isPci(void)
{
    return (dsPortEth == DS_PORT_PCI);
}

/******************************************************************************
 *
 * Name: dsPort_Write
 *
 * Description:
 *   This routine writes a buffer to the dsport.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_Write( apio_bufdescr_t *pBufDescr)
{
    if (dsPortEth)
        return (apio_etherWrite(pBufDescr));
    else
        return (apio_pciWrite(pBufDescr));
}

/******************************************************************************
 *
 * Name: dsPort_Read
 *
 * Description:
 *   This routine reads a buffer from the dsport.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   apio_bufdescr_t *pBufDescr
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_Read(apio_bufdescr_t *pBufDescr)
{
    if (dsPortEth)
        return (apio_etherRead(pBufDescr));
    else
        return (apio_pciRead(pBufDescr));
}

/******************************************************************************
 *
 * Name: dsPort_PowerSense
 *
 * Description:
 *   This routine reads the GPIO power sense bit..
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   nonzero - power active
 *   0       - power inactive
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
Timer pci_timer;
void pci_timer_process(void)
{
     etherBugSend("reg 0x80000010 = %x\n",WL_REGS32(0x80000010)); 
     if(WL_REGS32(0x80000010) == 0xd0000000 )
     {
         //apctlGPIOSetMode(13, 0);
         msi_wl_SetGPIOMode(13, 0);
         apctlGPIOWrite(13, 0);

     }
     return;
}
extern UINT32 dsPort_PowerSense(void)
{
#ifndef AP_CARD
    static UINT32 previous_status = 0xffffffff;
    pci_power = msi_wl_ReadGPIO(DS_GPIO_POWER_SENSE_BIT);
    if(previous_status == 0xffffffff)
        previous_status = pci_power;
    if ( previous_status != pci_power ) {
        previous_status = pci_power;

        if ( (!pci_power) && (!dsPortEth) ) {
            dsPort_SetEth(1);
        }
        if (pci_power)
        {
            TimerFireIn (&pci_timer, 1, &pci_timer_process, NULL, 10 * 10 );
        }
        return 1;
    }
#endif
    return 0;
}

/******************************************************************************
 *
 * Name: dsPort_ReadEthernetControl
 *
 * Description:
 *   This routine reads the GPIO Ethernet control bit..
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   nonzero - ethernet active
 *   0       - ethernet inactive
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_ReadEthernetControl(void)
{
    return (msi_wl_ReadGPIO(DS_GPIO_ETHER_CONTROL_BIT));
}


/*=============================================================================
 *                         CODED PRIVATE PROCEDURES
 *=============================================================================
 */

/******************************************************************************
 *
 * Name: dsPort_ControlEthernetEnable_PciDisable
 *
 * Description:
 *   This routine enables the dsport ethernet port and disables PCI access.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
static void dsPort_ControlEthernetEnable_PciDisable(void)
{
    apctlGPIOWrite(DS_GPIO_ETHER_CONTROL_BIT, DS_GPIO_ETHER_ENABLE);
}


/******************************************************************************
 *
 * Name: dsPort_ControlEthernetEnable_PciDisable
 *
 * Description:
 *   This routine disables the dsport ethernet port and enables PCI access.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 * 
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
static void dsPort_ControlEthernetDisable_PciEnable(void)
{
    apctlGPIOWrite(DS_GPIO_ETHER_CONTROL_BIT, DS_GPIO_ETHER_DISABLE);
}

#endif /*AP_PCI*/
