/*
*                Copyright 2002, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    oemflash.c
 * \brief   Flash driver routines for a number of flash types
 *
 * AUTHOR: cvu@marvell.com, created 12/2/2002
 * $Revision: 1.17 $
 * $Date: 2004/06/23 23:17:18 $
 */

/** include files **/
#ifdef ECOS
#include <cyg/kernel/kapi.h>
#endif
#include "wltypes.h"
#include "apcfg.h"
#include "flashdrvapi.h"
#include "osif.h"
#include "usrcfg.h"


extern int apctlIntDisable(void);
extern int apctlIntEnable(int);
extern void *malloc (size_t size);
extern void free (void *ptr);
extern void WatchdogDisable (void);
extern void WatchdogEnable (void);

/** local definitions **/

/* Manufacturer code */
#define AMD	        0x01
#define HYUNDAI 	0xAD
#define MICRONIX 	0xC2
#define SST         0xBF

#define _REG16(x) (*(volatile unsigned short int *)(x))

/* default settings */
#define SIZEOF_FLASH_SECTOR (64 * 1024)
#define SECTOR_OFFSET(sector) (sector * SIZEOF_FLASH_SECTOR)

/* AP Flash memory Map on 1MB flash part:
     Sector     Description
     ------     -----------
       15       Boot loader
       14       Factory and user configuration data (1KB each):
                   Offset 0..1023: factory settings
                   Offset 1024..2047: user settings
     13..0      Firmware image
*/
#ifdef ECOS
#define SECTOR_FACTORY_CONFIG (0xfffe0000 - flashbase)/SIZEOF_FLASH_SECTOR
#else /* VxWorks settings for now to avoid conflict with armboot. */
#define SECTOR_FACTORY_CONFIG (0xfffd0000 - flashbase)/SIZEOF_FLASH_SECTOR
#endif
#define OFFSET_FACTORY_CONFIG (SECTOR_FACTORY_CONFIG * SIZEOF_FLASH_SECTOR)
#define SIZE_FACTORY_CONFIG   1024

#define SECTOR_USER_CONFIG    SECTOR_FACTORY_CONFIG
#define OFFSET_USER_CONFIG    (OFFSET_FACTORY_CONFIG + SIZE_FACTORY_CONFIG)
#define SIZE_USER_CONFIG      SIZE_FACTORY_CONFIG

#define OFFSET_WIPOD_MODE     (OFFSET_FACTORY_CONFIG + SIZEOF_FLASH_SECTOR - 2)

/** external functions **/

/** external data **/

/** public data **/

/** private data **/
typedef struct FLASH_TYPE_t
{
    unsigned index;
    unsigned manu;
    unsigned addr1;
    unsigned addr2;
    unsigned val1;
}FLASH_TYPE_t;

#define MAX_VENDOR 4
FLASH_TYPE_t flash[] =
{
    {0, AMD, 0xAAA, 0x555, 0x0030},
    {1, HYUNDAI, 0XAAA, 0x555, 0x0030},
    {2, MICRONIX, 0xAAA, 0x555, 0x0030},
    {3, SST, 0xAAAA, 0x5554, 0x0050}
};
unsigned flashindex = 0;

unsigned int flashsize = 0;
unsigned int flashbase = 0xffff0000;
unsigned int flashNsect = 0;
unsigned int manu = 0;
unsigned int part = 0;

static int flashDrvInitialized = FALSE;
static int split_sector_kb = 0;

static int split_sector = -1;
#ifdef ECOS
static cyg_mutex_t writeFlash_mutex;
#endif

#define NUM_SUBSECTORS 4
static int split_sector_data[NUM_SUBSECTORS] = {
            0,
            0x8000,
            0xa000,
            0xc000,
        };

static int split[2][4] = {
                             { 16, 8, 8, 32},
                             { 32, 8, 8, 16}
                         };

/* A buffer to store the factory settings */
static char mFactorySettings[SIZEOF_MIB_USER_AP];
/** private functions **/

/**
 * Flash write or erase preamble for AMD part
 */
inline void
flashPreamble(void)
{
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
    _REG16(flashbase | flash[flashindex].addr1) = 0x0080;
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;
}

/**
 * Build split sector configuration data 
 * 
 * @param ss
 */
void
makesplit(unsigned ss)
{
    int top = 0;
    int start = 0;
    int i;

    split_sector = ss;

    if (ss)
    {
        top = 1;
    }

    for (i = 0; i < 4; ++i)
    {
        split_sector_data[i] = start;
        start += split[top][i] * 1024;
    }
}

/**
 * Interrogate flash part for manufacturer info and part info
 */
void
flashInterrogate(void)
{
    int i;
    for ( i = 0; i<MAX_VENDOR; i++ ) {
        _REG16(flashbase) = 0xf0; /* reset */

        _REG16(flashbase | flash[i].addr1) = 0x00AA;  /* word address 555 */
        _REG16(flashbase | flash[i].addr2) = 0x0055;  /* word address 2AA */
        _REG16(flashbase | flash[i].addr1) = 0x0090;  /* word address 555 */
        manu = _REG16(flashbase);     /* Manu */
        part = _REG16(flashbase + 2); /* part ID. */
        _REG16(flashbase) = 0xf0; /* reset */
        if ( manu == flash[i].manu ) {
            flashindex = i;
            return;
        }
    }
    flashindex = 0;
}

/**
 * Erase a block of sectors
 *
 * @param sblk   Start sector number
 * @param nblks  Number of sectors
 */
void
EraseSectors(int sblk, int nblks)
{
    unsigned addr, start;
    int i;
    int j;

    if ( !flashDrvInitialized ) {
        return;
    }

    for ( i = sblk;i < (sblk + nblks);i++ ) {
        addr = flashbase + SECTOR_OFFSET(i);
        if ( i != split_sector ) 
        {
            flashPreamble();
            _REG16(addr) = flash[flashindex].val1;
            while ( _REG16(addr) != 0xFFFF );
        }
        else 
        {
            start = addr;
            if ( split_sector_kb ) 
            {
                for ( j=0; j<(SIZEOF_FLASH_SECTOR/(split_sector_kb *1024)); j++ ) {
                    addr = start + j * (split_sector_kb *1024); // 8kb
                    flashPreamble();
                    _REG16(addr) = flash[flashindex].val1;
                    while ( _REG16(addr) != 0xFFFF );
                }
            }
            else 
            {
                for ( j = 0; j < NUM_SUBSECTORS; j++ ) {
                    addr = start + split_sector_data[j];
                    flashPreamble();
                    _REG16(addr) = flash[flashindex].val1;
                    while ( _REG16(addr) != 0xFFFF );
                }
            } // if split 8kb
        } // if split
    } // for
    return; 
}

/**
 * Write a word (16 bit)
 *
 * @param offset The offset of the address relative to the start of the flash
 * @param value  The value to write
 * @return TRUE: success
 *         FALSE: failure
 */
int
flashWriteWord( int offset, unsigned short dat)
{

    unsigned addr = flashbase | offset;
    unsigned short tmp;

    if (!flashDrvInitialized)
    {
        return FALSE;
    }

    /*flags = apctlIntDisable(); */
    _REG16(flashbase | flash[flashindex].addr1) = 0x00AA;  /* word address 555 */
    _REG16(flashbase | flash[flashindex].addr2) = 0x0055;  /* word address 2AA */
    _REG16(flashbase | flash[flashindex].addr1) = 0x00A0;  /* word address 555 */

    /* if flash is not erase we don't write (avoid infinit loop) */
    /* we can change a bit from 1 to 0 but not from 0 to 1 */
    tmp = _REG16 (addr);
    if ((~tmp & dat) != 0x00000000)
        return FALSE;   /* flash not erased */

    _REG16(addr) = dat;

    /* spin here until operation is done */
    while (_REG16(addr) != dat)
        ;


    return TRUE;
}

/******************************************************************************
 *
 * Name: EraseMfgData
 *
 * Description:
 *    Routine to erase block of memory from the flash
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (i  ): starting block
 *    Arg2 (i  ): number of block
 *
 * Return Value:
 *    TRUE is success.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
int EraseMfgData (int sblk, int nblks)
{
    unsigned short *src;
    unsigned short *dest;
    unsigned long nbytes;
    unsigned long offset;
    char *ptr;
    int status = TRUE;

    ptr = malloc(64 * 1024);
    if (ptr)
    {
        WatchdogDisable();
        nbytes = 64 * 1024;
        src = (unsigned short *)(mfgDataAddress() & 0xffff0000L);
        dest = (unsigned short *)ptr;

        while (nbytes)
        {
            *dest++ = *src++;
            nbytes -= sizeof(UINT16);
        }
        memset(ptr + (mfgDataAddress() & 0x0000ffffL), 0xff, 72);
        EraseSectors(sblk, nblks);
        src = (unsigned short *)ptr;
        nbytes = 64 * 1024;
        /*     offset = 0xffff0000; */
        offset = SECTOR_OFFSET (sblk) ;  /* so offset + flashbase = 0xFFFF0000 */
        while ((nbytes != 0) && (status == TRUE))
        {
            status = flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }
        free(ptr);
        WatchdogEnable();
    }

    return (status);
}

/**
 * Erase a sector
 * 
 * @param sectorNum The sector number
 * @return TRUE on success
 *         FALSE on failure
 */
static int
EraseFlashSector(int sectorNum)
{
    if (!flashDrvInitialized || (sectorNum >= flashNsect))
    {
        return FALSE;
    }

    EraseSectors(sectorNum, 1);

    return TRUE;
}

/**
 * Read factory settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApReadFactorySettings(mib_AP_t *factoryData)
{
    int nbytes;
    unsigned short *src;
    unsigned short *dest;

    if (!flashDrvInitialized || (factoryData == NULL))
    {
        return FALSE;
    }

    nbytes = SIZEOF_MIB_AP;
    src = (unsigned short *)(flashbase + OFFSET_FACTORY_CONFIG);
    dest = (unsigned short *)factoryData;

    while (nbytes)
    {
        *dest++ = *src++;
        nbytes -= sizeof(UINT16);
    }

    return TRUE;
}

/**
 * Read user settings from flash
 * 
 * @param factoryData
 *               Pointer to the buffer to receive the results
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApReadUserSettings(mib_AP_t *userData)
{
    int nbytes;
    unsigned short *src;
    unsigned short *dest;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }

#ifdef ECOS
    cyg_mutex_lock( &writeFlash_mutex );
#endif

	nbytes = SIZEOF_MIB_USER_AP;
    src = (unsigned short *)(flashbase + OFFSET_USER_CONFIG);
    dest = (unsigned short *)userData;

    while (nbytes)
    {
        *dest++ = *src++;
        nbytes -= sizeof(UINT16);
    }

#ifdef ECOS
    cyg_mutex_unlock( &writeFlash_mutex );
#endif

    return TRUE;
}


/**
 * Write user configuration to flash
 * 
 * @param userData Pointer to user data
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApWriteUserSettings(mib_AP_t *userData) /* 2003/05/29, fred chang */
{
    int nbytes;
    unsigned short *src;
    int offset;

    if (!flashDrvInitialized || (userData == NULL))
    {
        return FALSE;
    }

    /* We are going to erase the sector, so must save the factory settings */
    /* because it is on the same sector as the user data */
    ApReadFactorySettings((mib_AP_t *)mFactorySettings);

#ifdef ECOS
    cyg_mutex_lock( &writeFlash_mutex );
#endif

    WatchdogDisable();

    os_EnterCriticalSection;
    
    EraseFlashSector(SECTOR_FACTORY_CONFIG);

    /* Now write the factory data */
    nbytes = SIZEOF_MIB_AP;
    src = (unsigned short *)mFactorySettings;
    offset = OFFSET_FACTORY_CONFIG;

    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }

    /* Now write the user data */
    nbytes = SIZEOF_MIB_USER_AP;
    src = (unsigned short *)userData;
    offset = OFFSET_USER_CONFIG;

    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }

#if defined(WIPOD)
    flashWriteWord(OFFSET_WIPOD_MODE, (!(userData->wipodMode&0x80)? 1:0));
#endif

    os_ExitCriticalSection;
    WatchdogEnable();
#ifdef ECOS
    cyg_mutex_unlock( &writeFlash_mutex );
#endif
    return TRUE;
}

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
static int
ApResetToFactorySettings(void)
{
    int rc;

    rc = FALSE;

    memset(mFactorySettings, 0xff, sizeof(mFactorySettings));

    /* Read the factory default configuration from flash memory */
    if (ApReadFactorySettings((mib_AP_t *)mFactorySettings))
    {
#if defined(WIPOD)
        /* Duplicate for WB */
        memcpy(((mib_AP_t *)mFactorySettings) + 1, mFactorySettings, sizeof(mib_AP_t));
#endif
        rc = ApWriteUserSettings((mib_AP_t *)mFactorySettings);

    }

    return rc;
}
static int
ApCheckImageIntegrity(char *pData, unsigned long len)
{
    UINT32 *tmp, checksum, i;
    tmp = (UINT32 *)pData;

    if (*tmp != 0x4c56524d)
    {
#ifdef AP31HOST
        if(*tmp == 0x54534f48)
        {
            *tmp = 0x4c56524d;
            return 1;
        }
        else
#endif
        {
            /*wrong signature */
            if ((int) ((UINT32 *) 0xfffffff3) == 0xffffffff)
                return 1;  /*old bootcode, let it pass */
            return 0;
        }
    }
    checksum = *tmp;
    len = *(++tmp);
    for (i = 0; i < len + 8; i += 4)
    {
        checksum += *tmp++;
    }
    checksum = ~checksum;
    /*checksum should be 0 */
    return (!checksum);
}

#if defined(WIPOD)

typedef struct wpImg_t {
    char *pImg[2]; 
    int  imgLen[2];
}wpImg_t;

/**
 * Extract current boot code version
 * 
 * @return 0xaabbccdd = Numerical version of 0.0.0.0 - 9.9.9.9
 *          
 */
static UINT32
bootVersion(void)
{
    int i;
    UINT32 vers = 0;
    UINT8 *tmp = (UINT8 *) 0xfffffff4;

    for (i = 3;i >= 0; i--) {
        vers |= (*tmp - 0x30) << i*8;
        tmp += 2;
    }
    return (vers);
}

/**
 * Check WiPod image integrity and decapsulate individual images
 * 
 * @return Number of images decapsulated
 *          
 */
static int 
wpDecapImage(char *pData, unsigned long dataLen, wpImg_t *wpImg)
{
    UINT32 *tmp, chksum, i;
    UINT32 bootVers, len;

    if(!(pData && wpImg && dataLen)) {
        return 0;
    }

    bootVers = bootVersion();
    tmp = (UINT32 *) pData;
    if ((*tmp != 0x5057524d)||(dataLen != *(tmp+1))
        ||(bootVers < 0x00000005)||(bootVers > 0x09090909)) {
        /* Older boot code does not support WiPod image */ 
        return 0;
    }

    chksum = *tmp;
    len = *(++tmp);
    for (i = 0; i < len - 4; i += 4) {
        chksum += *tmp++;
    }

    if (~chksum) {
        return 0;
    }

    /* For WiPod, we need both images valid. */ 
    for(i = 0; i < 2; i++) {
        wpImg->pImg[i] = pData + ((UINT32 *)pData)[5+i];
        wpImg->imgLen[i] = *((UINT32 *)(wpImg->pImg[i] + 4)) + 12; 

        if(!ApCheckImageIntegrity(wpImg->pImg[i],wpImg->imgLen[i])) {
            memset(wpImg,0,sizeof(wpImg_t));
            return 0;
        }
    }
    return 2;
}
#endif /* WIPOD */


#define ENABLE_BACKUP_CODE 1 
/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
#if 1/*bt_test */
extern int
#else
static int
#endif/*bt_test */
    ApUpgradeFirmware(char *pData, unsigned long len)
{
    unsigned long nbytes;
    unsigned short *src;
    unsigned long offset;

#if defined(WIPOD)
    wpImg_t wpImg = { 0 };
#endif

    UINT32 *tmp;
    if (!flashDrvInitialized || (pData == NULL))
    {
        return FALSE;
    }
    WatchdogDisable();

#ifdef ENABLE_BACKUP_CODE

#if !defined(WIPOD)
    if (!ApCheckImageIntegrity(pData, len))
    {
        WatchdogEnable();
        return FALSE;
    }
#else
    if ( *((unsigned long *)pData) == 0x5057524d
        && wpDecapImage(pData,len,&wpImg) == 2 )
    {
        pData = wpImg.pImg[0];
        len = wpImg.imgLen[0];
    } 
    else 
    {
        WatchdogEnable();
        return FALSE;
    }
#endif /* WIPOD */

    /* only erase sector 7 - 13 for backup image */
    /* if no valid image here */
    tmp = (UINT32 *)0xfff70000;
    if (*tmp != 0x4c56524d )
    {
        EraseSectors(7, SECTOR_FACTORY_CONFIG - 7);

#if !defined(WIPOD)
        src = (unsigned short *)pData;
        nbytes = len;
        offset = 0x70000;
        if ((int) (UINT32 *) 0xfffffff3 == 0xffffffff) /*old bootcode */
        {
            src += 8;
            nbytes -= 8;
        }
        while (nbytes)
        {
            flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }
#else

        /* We have no backup image. Upgrade Img2 safely */
        src = ((unsigned short *)wpImg.pImg[1]) + 2;
        nbytes = wpImg.imgLen[1] - 4;
        offset = 0x70004;
        while (nbytes)
        {
            flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }

        src = (unsigned short *)wpImg.pImg[1];
        flashWriteWord(0x70000, *src);
        ++src;
        flashWriteWord(0x70002, *src);

        /* No need to upgrade again below */
        wpImg.pImg[1] = NULL;
#endif /* WIPOD */
    }

    /*then erase 0 - 6 for upgrade image */
    EraseSectors(0, 7);

    src = (unsigned short *)pData;
    if ((int) (UINT32 *) 0xfffffff3 == 0xffffffff) /*old bootcode */
    {
        src += 8;
    }
    src += 2;     /*leave out signature for later */
    nbytes = len - 4;

    offset = 4;
    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }
    /* write the signature now */
    src = (unsigned short *)pData;
    flashWriteWord(0, *src);
    ++src;
    flashWriteWord(2, *src);

#if defined(WIPOD)
    if(wpImg.pImg[1])
    {
        EraseSectors(7, SECTOR_FACTORY_CONFIG - 7);

        src = ((unsigned short *)wpImg.pImg[1]) + 2;     /*leave out signature for later */
        nbytes = wpImg.imgLen[1] - 4;
        offset = 0x70004;

        while (nbytes)
        {
            flashWriteWord(offset, *src);
            ++src;
            offset += sizeof(UINT16);
            nbytes -= sizeof(UINT16);
        }

        src = (unsigned short *)wpImg.pImg[1];
        flashWriteWord(0x70000, *src);
        ++src;
        flashWriteWord(0x70002, *src);
    }
#endif /* WIPOD */

#else
    EraseSectors(0, SECTOR_FACTORY_CONFIG - 1);

    src = (unsigned short *)pData;
    nbytes = len;
    offset = 0;
    while (nbytes)
    {
        flashWriteWord(offset, *src);
        ++src;
        offset += sizeof(UINT16);
        nbytes -= sizeof(UINT16);
    }
#endif /* ENABLE_BACKUP_CODE*/
    WatchdogEnable();

    return TRUE;
}

/** public functions **/

/**
 * User supplied function to initialize Flash driver struct
 * At startup time, the system will call this function to initialize
 * the flash driver.
 * 
 * @param apfdDriver_t
 *               Pointer to Flash Driver init struct.
 *               The initialized flash driver stuct is returned.
 */

void
apfdDriverInit(apfdDriver_t *apfdDrv)
{
    if (apfdDrv != NULL)
    {
        apfdDrv->WordMode = TRUE;
        apfdDrv->wrfunc.WrWord = flashWriteWord;
        apfdDrv->EraseSector = EraseFlashSector;
        apfdDrv->ReadFactorySettings = ApReadFactorySettings;
        apfdDrv->ReadUserSettings = ApReadUserSettings;
        apfdDrv->WriteUserSettings = ApWriteUserSettings;
        apfdDrv->ResetToFactorySettings = ApResetToFactorySettings;
        apfdDrv->UpgradeFirmware = ApUpgradeFirmware;
    }

    /* get info from the flash */
    flashInterrogate();

    /* is it a manufacturer we know and have we heard of the part? */
    switch (manu)
    {
    case AMD:
        switch ( part ) {
        case 0x22da:  /* LV800BT */
        case 0x22d6: // F800T
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(15);
            break;
        case 0x225b:  /* LV800BB */
        case 0x2258: // F800B
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(0);
            break;
        case 0x38: // LV081
        case 0xd5: // F080
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(-1);
            break;
        case 0x22c4:  /* 160BT */
        case 0x22d2: // F160T
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(31);
            break;
        case 0x2249:  /* 160BB */
        case 0x22d7: // LV160EB
        case 0x22d8: // F160B
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(0);
            break;
        case 0xc8: // LV017
        case 0xad: // F016
        case 0x3d: // F017
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(-1);
            break;
        case 0x22f6: // LV320DT
        case 0x22a7: // LV320AT
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(63);
            split_sector_kb = 8;
            break;
        case 0x22f9: // LV320DB
        case 0x22a8: // LV320AB
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(0);
            split_sector_kb = 8;
            break;
        default:
            return ;
        }
        break;
    case HYUNDAI:  /* Hyundai */
        switch ( part ) {
        case 0xD5:  /* 29F080 */
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            break;
        default:
            return ;
        }
        break;
    case SST:  /* SST */
        split_sector_kb = 4;
        switch ( part ) {
        case 0x2780: /* SST39LF/VF400A */
        case 0xd7:   /* SST39LF/VF040 */
            flashsize = 512 * 1024;
            flashNsect = 8;
            makesplit(-1);
            break;
        case 0x2781:  /* SST39VF800A */
        case 0xd8:    /* SST39LF/VF080 */
            flashsize = 1 * 1024 * 1024;
            flashNsect = 16;
            makesplit(-1);
            break;
        case 0x2782:  /* SST39VF160A */
        case 0xd9:    /* SST39LF/VF160 */
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(-1);
            break;
        case 0x2783:  /* SST39VF320A */
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(-1);
            break;
        default:
            return ;
        }
        break;
    case MICRONIX:
        switch ( part ) {
        case 0x22c4:  /* 160BT */
            flashsize = 2 * 1024 * 1024;
            flashNsect = 32;
            makesplit(31);
            break;
        case 0x22a7:  /* 160BT */
            flashsize = 4 * 1024 * 1024;
            flashNsect = 64;
            makesplit(-1);
            split_sector_kb = 8;
            break;
        default:
            return ;
        }
        break;
    default:
        return ;
    }

    flashbase = (0 - flashsize);

    flashDrvInitialized = TRUE;
}

void WaitForFlashReady(void)
{
#ifdef ECOS
    cyg_mutex_lock( &writeFlash_mutex );
    cyg_mutex_unlock( &writeFlash_mutex );
#endif
}
